module Recurso
  module Controller
    class Recurso::Forbidden < StandardError; end

    def self.included(base)
      base.helper_method :policy if base.respond_to?(:helper_method)
    end

    def authorize(resource, action = default_authorize_action)
      raise Recurso::Forbidden unless policy(resource).public_send(action)
    end

    def policy(resource = default_policy_resource)
      return Recurso::NilClassPolicy.new unless default_policy_identity.present?

      default_policy_identity.policy(resource)
    end

    private

    def default_authorize_action
      :view?
    end

    def default_policy_identity
      raise NotImplentedError.new("Please define a default policy identity with Recurso::Controller")
    end

    def default_policy_resource
      nil
    end
  end
end
